/******************************************************************
 *
 * Copyright 2015 Samsung Electronics All Rights Reserved.
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ******************************************************************/

#ifndef COMMON_UTIL_UNITTESTHELPERWITHFAKEOCPLATFORM_H
#define COMMON_UTIL_UNITTESTHELPERWITHFAKEOCPLATFORM_H

#include <gtest/gtest.h>
#include <HippoMocks/hippomocks.h>

#include "octypes.h"
#include "OCPlatform.h"

using namespace std;
using namespace std::placeholders;

/**
 * @class FakeOCPlatform
 */
class FakeOCPlatform
{
public:
    /** API is to subscribe the resource presence */
    virtual OCStackResult subscribePresence(OC::OCPlatform::OCPresenceHandle& a,
                                            const std::string& b, OCConnectivityType c,
                                            OC::SubscribeCallback d);

    /** API is to subscribe presence2 */
    virtual OCStackResult subscribePresence2(OC::OCPlatform::OCPresenceHandle& a,
                                             const std::string& b, const std::string& c,
                                             OCConnectivityType d, OC::SubscribeCallback e);

    /** API is to unsubscribe resource presence */
    virtual OCStackResult unsubscribePresence(OC::OCPlatform::OCPresenceHandle a);

    /** API is to find the resource */
    virtual OCStackResult findResource(const std::string& a, const std::string& b,
                               OCConnectivityType c, OC::FindCallback d);
    /** API is to register the resource */
    virtual OCStackResult registerResource(OCResourceHandle&, string&, const string&, const string&,
               OC::EntityHandler, uint8_t ) = 0;
    /** This method is for unregister resource */
    virtual OCStackResult unregisterResource(const OCResourceHandle &) = 0;

    /** This method is for send response */
    virtual OCStackResult sendResponse(const std::shared_ptr<OC::OCResourceResponse>) = 0;
    /** This method is to bind interface to resource */
    virtual OCStackResult bindInterfaceToResource(
            const OCResourceHandle &, const std::string &) = 0;
    /** This method is to bind the type to resource */
    virtual OCStackResult bindTypeToResource(const OCResourceHandle &, const std::string &) = 0;
    /** This method is for bind the resource */
    virtual OCStackResult bindResource(const OCResourceHandle, const OCResourceHandle) = 0;
    /** notify all observers */
    virtual OCStackResult notifyAllObservers(OCResourceHandle) = 0;

    virtual ~FakeOCPlatform() { }
};

extern MockRepository mocks;
extern FakeOCPlatform * mockFakePlatform;

/** namespace */
namespace OC
{
    namespace OCPlatform
    {
        /** This method is for subscribe the resource presence */
        OCStackResult subscribePresence(OCPresenceHandle& a, const std::string& b,
                                        OCConnectivityType c, SubscribeCallback d);

        /** This method is for subscribe presence */
        OCStackResult subscribePresence(OCPresenceHandle& a, const std::string& b,
                                        const std::string& c, OCConnectivityType d,
                                        SubscribeCallback e);

        /** This method is forunsubscribe presence */
        OCStackResult unsubscribePresence(OCPresenceHandle a);

        /** This method is for find resource */
        OCStackResult findResource(const std::string& a, const std::string& b,
                                   OCConnectivityType c, FindCallback d);

        /** This method is for register resource */
        OCStackResult registerResource(
                OCResourceHandle &a, string &b, const string &c,
                const string &d, EntityHandler e, uint8_t f);

        /** This method is for unregister resource */
        OCStackResult unregisterResource(const OCResourceHandle &a);

        /** This method is for bind resource */
        OCStackResult bindResource(const OCResourceHandle a, const OCResourceHandle b);

        /** This method is for send response */
        OCStackResult sendResponse(const std::shared_ptr<OCResourceResponse> a);

        /** This method is for bind the type to resource */
        OCStackResult bindTypeToResource(const OCResourceHandle &a,
                                const std::string &b);

        /** This method is for bind the interface to resource */
        OCStackResult bindInterfaceToResource(const OCResourceHandle &a,
                                              const std::string &b);

        /** This method is for notify to all observers */
        OCStackResult notifyAllObservers(OCResourceHandle a);
    }
}

/**
 * @class TestWithExternMock
 */
class TestWithExternMock: public testing::Test
{
protected:
    virtual ~TestWithExternMock() noexcept(noexcept(std::declval<Test>().~Test())) {}

    virtual void SetUp()
    {
        mockFakePlatform = mocks.Mock< FakeOCPlatform >();
    }

    virtual void TearDown()
    {
        mocks.reset();
    }
};

#endif // COMMON_UTIL_UNITTESTHELPERWITHFAKEOCPLATFORM_H
